/*
***********************************************************************
China Mine version 1.1
Copyright (C) 1998, 1999, 2000  Ma Guokai, from People's Republic of China

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If what you get is only part of the program, send an email to akey@fudan.edu
give the name of the program and the version number (newest by default), I'll
send you a copy of the full package.
*****************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "http.h"
#include "counter.h"
#include "register.h"
#include "misc.h"
#include "recordFile.h"

#define REPLACE "%Replace%"

static int isHttp=0;

void writeTextContent()
{
    puts("Content-Type: text/plain");
	puts("Expires: Tue, 20 Aug 1996 14:25:27 GMT\n");
	isHttp=0;
}

void writeHtmlContent()
{
    puts("Content-Type: text/html");
	puts("Expires: Tue, 20 Aug 1996 14:25:27 GMT\n");
	isHttp=1;
}

void writeHtml(char *title, char *content)
{
	puts("<html><head><title>");
	puts(title);
	puts("</title></head><body>");
	puts(content);
	puts("</body></html>");
	exit(0);
}

void showError(char* errorMessage)
{
	if(!isHttp)
	{
		puts("+err");
		puts(errorMessage);
		exit(0);
	}
	else
	{
		puts("<br>Error!<br>");
		puts(errorMessage);
		exit(0);
	}
}

static void convertString(char* string)
{
	int i=0;
	int j;
	while(*string)
	{
		if(i>bufferSize-50)// leave out at least 48 chars
			break;
		switch(*string)
		{
			case '\n':
				buffer[i++]='<';
				buffer[i++]='b';
				buffer[i++]='r';
				buffer[i++]='>';
				break;
			case '<':
				buffer[i++]='&';
				buffer[i++]='l';
				buffer[i++]='t';
				buffer[i++]=';';
				break;
			case '>':
				buffer[i++]='&';
				buffer[i++]='g';
				buffer[i++]='t';
				buffer[i++]=';';
				break;
			case '&':
				buffer[i++]='&';
				buffer[i++]='a';
				buffer[i++]='m';
				buffer[i++]='p';
				buffer[i++]=';';
				break;
			case ' ':
				buffer[i++]='&';
				buffer[i++]='n';
				buffer[i++]='b';
				buffer[i++]='s';
				buffer[i++]='p';
				buffer[i++]=';';
				break;
			case '\t':
				for(j=0;j<8;j++)
				{
					buffer[i++]='&';
					buffer[i++]='n';
					buffer[i++]='b';
					buffer[i++]='s';
					buffer[i++]='p';
					buffer[i++]=';';
				}
				break;
			case '\"':
				buffer[i++]='&';
				buffer[i++]='q';
				buffer[i++]='u';
				buffer[i++]='o';
				buffer[i++]='t';
				buffer[i++]=';';
				break;
			default:
				buffer[i++]=*string;
		}
		string++;
	}
	buffer[i]='\0';
}

static string cgi_location;
static string* files;
static int fileNumber;
static string levelName[4];
static string tagName[2];
static string tagImage[2];
static string weekName[7];
static string nullString;

static void processCounter(char* start);
static void processBest(char* start);
static char* getWeekDay(int day);
static void readIni(void);

void showBestTime()
{
	int fileNo=0;
	FILE *fp;
	char buf[bufferSize];
	char *position, *oldPosition;

	writeHtmlContent();
	if(argc<2)
		writeHtml("Error!", "Missing parameter");
	readIni();
	if(argc==3)
	{
		fileNo=atoi(argv[2]);
		if(fileNo>=fileNumber || fileNo<0)
			writeHtml("Error!", "File index out of range.");
	}
	makeFilename(files[fileNo]);
	if(!(fp=fopen(buffer, "rb")))
		writeHtml("Error!", "Cannot open the specified file index.");
	while(fgets(buf, bufferSize, fp))
	{
		position=buf;
		while(1)
		{
			oldPosition=position;
			position=strstr(oldPosition, REPLACE);
			if(position)
				*position='\0';
			fputs(oldPosition, stdout);
			if(!position)
				break;
			position=position+strlen(REPLACE);
			switch(*position)
			{
				case 'A': /* ip address */
					printf(cgi_location);
					position+=1;
					continue;
				case 'C': /* counter class */
					processCounter(position+1);
					position+=2;
					continue;
				case 'B': /* best class */
					processBest(position+1);
					position+=4;
					continue;
				default:
					showError("Parse error, can't recognize replace alias.");
			}
		}
	}
	exit(0);
}

static void processCounter(char* start)
{
	static int first=1;

	if(first)
		getCounter(0);
	first=0;
	switch(*(start))
	{
		case '1': /* total counts */
			printf("%d", countNumber);
			break;
		case '2': /* counts today */
			printf("%d", countNumberToday);
			break;
		case '3': /* year */
			printf("%d", year+1900);
			break;
		case '4': /* month */
			printf("%d", month+1);
			break;
		case '5': /* day */
			printf("%d", day);
			break;
		case '6': /* hour */
			printf("%d", hour);
			break;
		case '7': /* minute */
			printf("%02d", minute);
			break;
		case '8': /* week day */
			printf(getWeekDay(weekDay));
			break;
		default:
			showError("Parse error, can't recognize replace alias.");
	}
}

static int outputRecord(int level, int index);
static char* getLevel(int level);
static char* getTagImage(int tag);
static char* getTagName(int tag);
static void processBest(char* start)
{
	winnerData temp;
	FILE *fp;
	static int first=1;
	byte current;
	int i;

	if(first)
		checkFiles();
	first=0;
	switch(*start)
	{
		case '1': /* best */
		case '2': /* best today */
			if((*start)=='1')
				fp=fpAll;
			else
				fp=fpToday;
			if(*(start+1)<'1' || *(start+1)>'8')
				showError("Parse error, can't recognize replace alias.");
			if(readRecord(fp, 0, (*(start+1))-'1', &temp))
				showError("Read error, cannot read file");
			if(temp.levelNtag==255)
			{
				printf("/"); /* N/A */
				break;
			}
			switch(*(start+2))
			{
				case '1': /* level */
					printf(getLevel(temp.levelNtag%4));
					break;
				case '2': /* tag */
					printf("<img src=\"%s\" alt=\"%s\">",
							getTagImage(temp.levelNtag/4),
							getTagName(temp.levelNtag/4));
					break;
				case '3': /* score */
					printf("%d", temp.score);
					break;
				case '4': /* year */
					printf("%d", temp.year+1900);
					break;
				case '5': /* month */
					printf("%d", temp.month+1);
					break;
				case '6': /* day */
					printf("%d", temp.day);
					break;
				case '7': /* hour */
					printf("%d", temp.hour);
					break;
				case '8': /* minute */
					printf("%02d", temp.minute);
					break;
				case '9': /* name */
					convertString(temp.name);
					printf("%s", buffer);
					break;
				case '0': /* comment */
					convertString(temp.comment);
					printf("%s", buffer);
					break;
				case 'a': /* weekday */
					printf(getWeekDay(temp.weekDay));
					break;
				default:
					showError("Parse error, can't recognize replace alias.");
			}
			break;
		case '3': /* history */
			fp=fpList;
			if(*(start+1)<'1' || *(start+1)>'4')
				showError("Parse error, can't recognize replace alias.");
			if(fseek(fp, *(start+1)-'1', SEEK_SET))
				showError("Read error, cannot seek file");
			if(fread(&current, 1, 1, fp)!=1)
				showError("Read error, cannot read file");
			if(current>=100)
				showError("File error, the current point corrupted");
			for(i=0;i<100;i++)
			{
				if(outputRecord(*(start+1)-'1', (current+100-i)%100 /* it will not overflow */))
					break;
				puts("<br>");
			}
			if(!i)
				puts(nullString);
			if(readRecord(fp, 0, (*(start+1))-'1', &temp))
				showError("Read error, cannot read file");
			break;
	}
}

static char* getLevel(int level)
{
	if(level<0 || level>=4)
		showError("Parse error, can't recognize replace alias.");
	return levelName[level];
}

static char* getTagImage(int tag)
{
	return tagImage[tag?1:0];;
}

static char* getTagName(int tag)
{
	return tagName[tag?1:0];
}

static char* getWeekDay(int day)
{
	if(day<0 || day>=7)
		showError("Not a week day.");
	return weekName[day];
}

static int outputRecord(int level, int index)
{
	winnerData tempRecord;
	if(readRecord(fpList, 4, level*historyLength+index, &tempRecord))
		showError("Read error, cannot read file");
	if(tempRecord.levelNtag==255)
		return -1;
	puts("<table width=\"75%\" border=\"0\" cellspacing=\"0\" align=\"center\">\n<tr>");
	printf("<td width=\"10%%\" bgcolor=\"#FFFFEE\">%s</td>\n",getLevel(tempRecord.levelNtag%4));
	printf("<td width=\"90%%\" bgcolor=\"#DDEEFF\">ǣ<img src=\"%s\" alt=\"%s\"></td>\n",
			getTagImage(tempRecord.levelNtag/4), getTagName(tempRecord.levelNtag/4));
	puts("</tr>\n<tr>");
	printf("<td width=\"10%%\" bgcolor=\"#EEEEFF\">ɼ%d</td>\n", tempRecord.score);
	convertString(tempRecord.name);
	printf("<td width=\"90%%\" bgcolor=\"#EEFFEE\">%s</td>\n", buffer);
	puts("</tr>\n<tr bgcolor=\"#FFEEEE\">");
    printf("<td colspan=\"2\">ʱ䣺%d%d%dգ%s%d:%02d</td>\n", tempRecord.year+1900,
				tempRecord.month+1, tempRecord.day, getWeekDay(tempRecord.weekDay),
				tempRecord.hour, tempRecord.minute);
	puts("</tr>\n<tr bgcolor=\"#EEEEEE\">");
	puts("<td width=\"10%%\">ԣ</td>");
	convertString(tempRecord.comment);
	printf("<td width=\"90%%\">%s</td>", buffer);
	puts("</tr>\n</table>");
	return 0;
}

static void readIni()
{
	FILE *fp;
	int i, temp;
	makeFilename("mineCGI.ini");
	if(!(fp=fopen(buffer, "r")))
		showError("can not open file for read.");
	readBuffer(fp);
	cgi_location=newString(buffer);
	for(i=0;i<4;i++)
	{
		readBuffer(fp);
		levelName[i]=newString(buffer);
	}
	for(i=0;i<2;i++)
	{
		readBuffer(fp);
		tagName[i]=newString(buffer);
	}
	for(i=0;i<2;i++)
	{
		readBuffer(fp);
		tagImage[i]=newString(buffer);
	}
	for(i=0;i<7;i++)
	{
		readBuffer(fp);
		weekName[i]=newString(buffer);
	}
	readBuffer(fp);
	temp=atoi(buffer);
	if(temp<0 || temp>100)
		showError("wrong mineCGI.ini");
	files=(string*)malloc(sizeof(string)*temp);
	if(!files)
		showError("Not enough memory");
	for(i=0;i<temp;i++)
	{
		readBuffer(fp);
		files[i]=newString(buffer);
	}
	fileNumber=temp;
	readBuffer(fp);
	nullString=newString(buffer);
}

